"use strict";

this.name        = "ups_container";
this.author      = "eric walch";
this.copyright   = "2008 eric walch.";
this.license     = "CC BY-NC-SA 3.0";
this.description = "Container missions of UPS Courier";

this.startUp = function ()
{
    this.pirate = "pirate"; // since 1.73 pirates get a bounty
    this.tiger = (worldScripts.TigersTurf) ? "tigers" : this.pirate;
    this.upsMarkSystem = worldScripts["ups_docs"].upsMarkSystem;
    this.upsUnmarkSystem = worldScripts["ups_docs"].upsUnmarkSystem;
    if (worldScripts.BountySystem_Core) {
        var bsc = worldScripts.BountySystem_Core;
        bsc._offenceTypes["ups_policescanneruse"] = {
            description: "Using an illegal Police Scanner.",
		    severity: 1
        };
        bsc._offenceTypes["ups_idcleanerremoved"] = {
            description: "Caught with an illegal ID Cleaner on board ship.",
		    severity: 2
        };
        bsc._offenceTypes["ups_scannerremoved"] = {
            description: "Caught with illegal Police Scanner on board ship.",
		    severity: 2
        };
        bsc._offenceTypes["ups_piratelife"] = {
            description: "Fraternising with and actively supporting pirate activity.",
		    severity: 3
        };
    }
}

this.shipLaunchedFromStation = function ()
{
    this.setUpShips();
    if (this.ups_container === "SPILLAGE2")
    {
        this.ups_container = "SPILLAGE3";
        this.spillCargo();
    }
    this.startScanners();
}

this.missionScreenOpportunity = function ()
{
    if (player.ship.docked)
    {
        this.missionOffers();
    }
}

this.guiScreenChanged = function (newScreen, oldScreen)
{
    if (this.restartMissionOffer && newScreen !== "GUI_SCREEN_LONG_RANGE_CHART")
    {
        this.restartMissionOffer = false;
        if (guiScreen !== "GUI_SCREEN_MISSION") this.missionOffers();
    }
}

this.missionOffers = function ()
{
    if (this._savedTargetSet) player.ship.targetSystem = this._savedTarget
    this._savedTarget = null;
    this._savedTargetSet = false;

	var containerStatus = this.ups_container;
    if (player.ship.dockedStation.isMainStation)
    {
        var ups_choices;
        if (containerStatus === "BOA" && worldScripts["ups_parcel"].ups_pcount > 2 && system.government > 3
                        && player.ship.bounty < 3 && this._reputation() > 2 && !worldScripts["ups_docs"].lookLongRangeChartScreen)
        {
            ups_choices = "ups_container_accepted_yesnolook";
            if (!this.mark)
            {
                var infoList = this._getSystemsInRange(35);
                var targetSystem = infoList[Math.floor(Math.random()*infoList.length)];
                this.ups_cplanet = targetSystem.systemID;
                missionVariables.ups_cplanetname = targetSystem.name;
                this.ups_distance = system.info.distanceToSystem(targetSystem).toFixed(1)
                            + " LY to the "+this.positionToDirection(targetSystem.coordinates);
                this.mark = true;
            }
            var key;
			switch (this.ups_ccount)
			{
				case 0: key = "ups_container_1st_offer"; break;
				case 1: key = "ups_container_offer"; break;
				default: key = "ups_container_offer_bis"; break;
			}
            var message = expandMissionText(key, {ups_distance: this.ups_distance, ups_cplanet: this.ups_cplanet});
            mission.runScreen({screenID: "ups-container", title: "Boa Mission", message: message, overlay: this._overlay(),
                                choicesKey: ups_choices, model: this.boa()}, this.choiceEvaluation);
            this.offering = "UPS_CONTAINER1";
            this.ups_container_count = 0;
            this.ups_container_award = 0;
        }
        if (containerStatus === "SEARCHING")
        {
            var daysPast = clock.days - this.ups_timerstart;
            if (daysPast > 30 && system.government > 1)
            {
                this.ups_container = "NOT_NOW";
                var message = expandMissionText("ups_container_boa_withdraw", {ups_days_past: daysPast});
                mission.runScreen({screenID: "ups-container", title: "Boa Mission", message: message, overlay: this._overlay()});
                delete this.ups_timerstart;
                this.upsUnmarkSystem(this.ups_cplanet, this.name);
                mission.setInstructionsKey(null);
            }
            else if (this.ups_cplanet === system.ID && (missionVariables.ups_closecontact === "DOCKED" || system.countShipsWithRole("boa_ptt") === 0))
            {
                this.upsUnmarkSystem(this.ups_cplanet, this.name);
                var infoList = this._getSystemsInRange(7);
                var info = infoList[Math.floor(Math.random()*infoList.length)];
                this.ups_cplanet = info.systemID;
                missionVariables.ups_cplanetname = System.systemNameForID(info.systemID);
                mission.runScreen({screenID: "ups-container", title: "Boa Mission", messageKey: "ups_container_missed", overlay: this._overlay()});
                mission.setInstructions(expandMissionText("ups_containers_small1b", {ups_cplanet: this.ups_cplanet}));
                this.upsMarkSystem(this.ups_cplanet, this.name);
				missionVariables.ups_closecontact = "NO";
            }
        }
        if (containerStatus === "FOUND" && system.government > 1)
        {
            this.ups_container = "NOT_NOW";
            this.upsUnmarkSystem(this.ups_cplanet, this.name);
            mission.setInstructionsKey(null);
            var key;
            if (!this.ups_container_count)
            {
                // no containers scooped.
                if (missionVariables.ups_closecontact === "YES")
                {
                    key = "ups_container_destroyed0";
                    player.credits += 100;
                }
                else
                {
                    key = "ups_container_destroyed0a";
                }
            }
            else
            {
                if (this.ups_container_count === 5)
                {
                    // all containers scooped.
                    this.ups_container_award += 1000;
                    key = "ups_container_destroyed2";
                }
                else
                {
                    // some containers scooped.
                    key = "ups_container_destroyed1";
                }
                this.ups_ccount++;
                player.credits += this.ups_container_award;
            }
            var message = expandMissionText(key, {ups_container_count: this.ups_container_count, ups_container_award: this.ups_container_award});
            mission.runScreen({screenID: "ups-container", title: "Boa Mission", message: message, overlay: this._overlay()});
            missionVariables.ups_target = "hostile Boa";
            if (!worldScripts.GNN && worldScripts.snoopers && worldScripts.snoopers.version >= 2.0) worldScripts.snoopers.insertNews({ID: this.name, Agency: 1, Message: expandDescription("[ups_target_killed]")});
            if (worldScripts.GNN && worldScripts.GNN.version >= 1.0) worldScripts.GNN._insertNews({ID: this.name, Agency: 1, Message: expandDescription("[ups_target_killed]")});
			this.ups_timerstart = clock.days;
			worldScripts["ups_docs"].increaseReputation(5);
            delete missionVariables.ups_closecontact;
            delete missionVariables.ups_b2planet; // no longer used variables
            delete missionVariables.ups_b2planetname; // no longer used variables
			if (this.ups_total_container_count < 5) // there are still containers out there.
			{
				this.ups_container = "SPILLAGE3"; // go further as spillage mission.
				this.ups_cplanet = system.ID;
				this.ups_container_remaining = system.countShipsWithPrimaryRole("ups-spilled-container");
				this.ups_container_count = 0;
				this.ups_container_award = 0;
			}
			else
			{
				delete this.ups_total_container_count;
			}
        }
        if (containerStatus === "LOGGING")
        {
            var daysPast = clock.days - this.ups_timerstart;
            if (system.government > 3 && player.ship.bounty < 3 && daysPast > 8
                        && this._reputation() > 3 && !worldScripts["ups_docs"].lookLongRangeChartScreen)
            {
                ups_choices = "ups_container_accepted2_yesnolook";
                var messageKey = (!this.ups_c1count ? "ups_container_1st_logging" : "ups_container_logging");
                var message = expandMissionText(messageKey, {ups_days_past: daysPast, ups_cplanet: this.ups_cplanet});
                mission.runScreen({screenID: "ups-container", title: "Python Mission", message: message,
                                    overlay: this._overlay(), choicesKey: ups_choices,
                                    model: this.python()}, this.choiceEvaluation)
                this.offering = "UPS_CONTAINER2";
                this.ups_container_count = 0;
                this.ups_container_award = 0;
            }
        }
        if (containerStatus === "FOUND2")
        {
            if (system.government > 1)
            {
                this.ups_container = "NOT_NOW";
                this.upsUnmarkSystem(this.ups_cplanet, this.name);
                mission.setInstructionsKey(null);
                var key;
                if (!this.ups_container_count)
                {
                    key = "ups_container_destroyed02";
                    player.credits += 100;
                }
                else
                {
                    if (this.ups_container_count === 5)
                    {
                        this.ups_container_award += 1000;
                        key = "ups_container_destroyed22";
                    }
                    else
                    {
                        key = "ups_container_destroyed12";
                    }
                    this.ups_c1count += 1;
                    player.credits += this.ups_container_award;
                }
                var message = expandMissionText(key, {ups_container_count: this.ups_container_count, ups_container_award: this.ups_container_award});
                mission.runScreen({screenID: "ups-container", title: "Python Mission", message: message, overlay: this._overlay()});
                delete this.ups_container_award;
                delete this.ups_container_count;
				worldScripts["ups_docs"].increaseReputation(3);
                if (this.ups_c1count > 4 && !missionVariables.TL_FOR_EQ_UPS_PRIVATE_ESCORT) {this.ups_container_message = 2};
                missionVariables.ups_target = "pirate python";
                if (!worldScripts.GNN && worldScripts.snoopers && worldScripts.snoopers.version >= 2.0) worldScripts.snoopers.insertNews({ID: this.name, Agency: 1, Message: expandDescription("[ups_target_killed]")});
                if (worldScripts.GNN && worldScripts.GNN.version >= 1.0) worldScripts.GNN._insertNews({ID: this.name, Agency: 1, Message: expandDescription("[ups_target_killed]")});
				this.ups_timerstart = clock.days;
                delete this.ups_container_destroyed;
            }
            else
            {
                if (!this.ups_container_destroyed)
                {
                    var message = expandMissionText("ups_container_destroyed3", {ups_container_count: this.ups_container_count, dockedStationName: player.ship.dockedStation.displayName});
                    mission.runScreen({screenID: "ups-container", title: "Python Mission", message: message, overlay: this._overlay()});
                    this.upsUnmarkSystem(this.ups_cplanet, this.name);
                    if (!this.ups_container_count) mission.setInstructionsKey("ups_containers_small2b")
                    else mission.setInstructionsKey("ups_containers_small2a");
                    this.ups_container_destroyed = true;
                }
            }
        }
        if (containerStatus === "LOGGING2")
        {
            var daysPast = clock.days - this.ups_timerstart;
            if (system.government > 3 && player.ship.bounty < 3 && daysPast > 8
                            && this._reputation() > 4 && !worldScripts["ups_docs"].lookLongRangeChartScreen)
            {
                ups_choices = "ups_container_accepted3_yesnolook";
                var key;
                if (player.ship.equipmentStatus("EQ_CLOAKING_DEVICE") === "EQUIPMENT_OK")
                {
                    key = (!this.ups_c2count) ? "ups_container_1st_logging2cloaked" : "ups_container_logging2cloaked";
                }
                else
                {
                    key = (!this.ups_c2count) ? "ups_container_1st_logging2" : "ups_container_logging2";
                }
                var message = expandMissionText(key, {ups_days_past: daysPast});
                mission.runScreen({screenID: "ups-container", title: "Pirate Base Mission", message: message, overlay: this._overlay(),
                                    choicesKey: ups_choices}, this.choiceEvaluation)
                this.offering = "UPS_CONTAINER3";
                this.ups_container_award = 0;
                this.ups_container_count = 0;
            }
        }
        if (containerStatus === "SEARCHING3" && this.ups_c2count > 2)
        {
            if (!missionVariables.ups_switch) missionVariables.ups_switch = this.ups_c2count
            else missionVariables.ups_switch += this.ups_c2count;
            if ((missionVariables.ups_switch - Math.random()*100) > 30 && system.ID !== this.ups_cplanet)
            {
                this.ups_container = "SEARCHING4";
                delete missionVariables.ups_switch;
                this.ups_timerstart = clock.days;
            }
        }
        if (containerStatus === "FOUND3" && system.government > 1)
        {
            this.ups_container = "NOT_NOW";
            this.upsUnmarkSystem(this.ups_cplanet, this.name);
            mission.runScreen({screenID: "ups-container", title: "Pirate Base Mission", messageKey: "ups_container_base_destroyed", overlay: this._overlay()});
            mission.setInstructionsKey(null);
            if (!this.ups_c2count) this.ups_c2count = 0;
            this.ups_c2count++;
            player.credits += 2000;
			worldScripts["ups_docs"].increaseReputation(5);
            delete missionVariables.ups_switch;
            missionVariables.ups_target = "pirate base";
            if (!worldScripts.GNN && worldScripts.snoopers && worldScripts.snoopers.version >= 2.0) worldScripts.snoopers.insertNews({ID: this.name, Agency: 1, Message: expandDescription("[ups_target_killed]")});
            if (worldScripts.GNN && worldScripts.GNN.version >= 1.0) worldScripts.GNN._insertNews({ID: this.name, Agency: 1, Message: expandDescription("[ups_target_killed]")});
			this.ups_timerstart = clock.days;
        }
        if (containerStatus === "SEARCHING4" && system.government > 3)
        {
            if (clock.days - this.ups_timerstart > 15)
            {
                mission.runScreen({screenID: "ups-container", title: "Pirate Base Mission", messageKey: "ups_container_qbomb", overlay: this._overlay()});
                this.ups_timerstart = clock.days;
            }
        }
        if (containerStatus === "FOUND4" && system.government > 1)
        {
            this.ups_container = "NOT_NOW";
            this.upsUnmarkSystem(this.ups_cplanet, this.name);
            mission.runScreen({screenID: "ups-container", title: "Pirate Base Mission", messageKey: "ups_container_base2_destroyed", overlay: this._overlay()});
            mission.setInstructionsKey(null);
            if (!this.ups_c3count) this.ups_c3count = 0;
            this.ups_c3count++;
            player.credits += 4000;
            if (this.ups_c3count == 1) this.ups_container_message = 1;
            if (this.ups_c3count > 1 && player.score > 6400 && !missionVariables.TL_FOR_EQ_UPS_UNIVERSAL_SCANNER)
            {
                this.ups_container_message = 3
            };
			worldScripts["ups_docs"].increaseReputation(6);
            delete missionVariables.ups_switch;
            missionVariables.ups_target = "guarded pirate base";
            if (!worldScripts.GNN && worldScripts.snoopers && worldScripts.snoopers.version >= 2.0) worldScripts.snoopers.insertNews({ID: this.name, Agency: 1, Message: expandDescription("[ups_target_killed]")});
            if (worldScripts.GNN && worldScripts.GNN.version >= 1.0) worldScripts.GNN._insertNews({ID: this.name, Agency: 1, Message: expandDescription("[ups_target_killed]")});
			this.ups_timerstart = clock.days;
        }
        if (containerStatus === "JOINED_PIRATES" && system.government > 3 && Math.random() < 0.2)
        {
            this.ups_container = "JOINED_PIRATES2";
            mission.runScreen({screenID: "ups-container", title: "Pirate Base Mission", messageKey: "ups_container_piratelife", overlay: this._overlay()});
            //player.ship.bounty += 1000;
            player.ship.setBounty(player.ship.bounty + 1000, "ups_piratelife");

            mission.setInstructionsKey(null);
            this.upsUnmarkSystem(this.ups_cplanet, this.name);
			worldScripts["ups_docs"].increaseReputation(-25);
         }
        if (containerStatus === "SPILLAGE" && system.government > 3)
        {
            this.ups_container = "SPILLAGE2";
            mission.runScreen({screenID: "ups-container", title: "Container Retreival", messageKey: "ups_container_scattered_1", overlay: this._overlay()});
            this.ups_cplanet = system.ID;
            this.upsMarkSystem(this.ups_cplanet, this.name);
            mission.setInstructionsKey("ups_container_scattered_small");
        }
        if (containerStatus === "SPILLAGE3")
        {
            if (this.ups_container_count > 0)
            {
                var message = expandMissionText("ups_container_scattered_2", {ups_container_count: this.ups_container_count, ups_container_award: this.ups_container_award});
                mission.runScreen({screenID: "ups-container", title: "Container Retreival", message: message, overlay: this._overlay()});
                this.ups_container_remaining = system.countShipsWithRole("ups-spilled-container");
                player.credits += this.ups_container_award;
				worldScripts["ups_docs"].increaseReputation(Math.ceil(this.ups_container_count/5));
                if (this.ups_container_remaining > 0)
                {
                    message = expandMissionText("ups_container_scattered_3", {ups_container_remaining: this.ups_container_remaining});
                    mission.addMessageText(message);
                    mission.setInstructionsKey("ups_container_scattered_small2");
					this.ups_container_award = 0;
					this.ups_container_count = 0;
					this.ups_total_container_count = 0;
                }
                else
                {
                    this.ups_container = "NOT_NOW";
                    this.upsUnmarkSystem(this.ups_cplanet, this.name);
                    mission.setInstructionsKey(null);
                    delete this.ups_cplanet;
					delete this.ups_container_remaining;
					delete this.ups_container_award;
					delete this.ups_container_count;
					delete this.ups_total_container_count;
                }
            }
        }
        if (this.ups_container_message > 0 && guiScreen !== "GUI_SCREEN_MISSION")
        {
            var replacements = {ups_doc_count: worldScripts["ups_docs"].ups_dcount, ups_parcel_count: worldScripts["ups_parcel"].ups_pcount,
                                ups_boa_count: this.ups_ccount, ups_python_count: this.ups_c1count, ups_base1_count: this.ups_c2count,
                                ups_base2_count: this.ups_c3count}; // used to update variables in the missiontext below.
            var message;
            if (this.ups_container_message === 1)
            {
                message = expandMissionText("ups_container_the_end", replacements);
                mission.runScreen({screenID: "ups-container", title: "Container Endmessage", message: message, overlay: this._overlay()});
                delete this.ups_container_message;
            }
            if (this.ups_container_message === 2 && system.government > 1 && Math.random() > 0.9)
            {
                message = expandMissionText("ups_container_message2", replacements);
                mission.runScreen({screenID: "ups-container", title: "UPS Escort Offer", message: message, overlay: this._overlay()});
                missionVariables.TL_FOR_EQ_UPS_PRIVATE_ESCORT = 7;
                delete this.ups_container_message;
            }
            if (this.ups_container_message === 3 && system.government > 1 && system.techLevel > 11 && Math.random() > 0.5)
            {
                message = expandMissionText("ups_container_message3", replacements);
                mission.runScreen({screenID: "ups-container", title: "UPS Scanner Offer", message: message, overlay: this._overlay()});
                missionVariables.TL_FOR_EQ_UPS_UNIVERSAL_SCANNER = 12;
                delete this.ups_container_message;
            }
        }
		if ((containerStatus === "NO" || containerStatus ==="NOT_NOW") && system.government > 1)
		{
			if (player.ship.equipmentStatus("EQ_UPS_POLICE_SCANNER") === "EQUIPMENT_OK" && Math.random() < 0.2)
			{
				player.ship.removeEquipment("EQ_UPS_POLICE_SCANNER");
				missionVariables.ups_galcop_punishment = 40;
				mission.runScreen({screenID: "ups-container", title: "Scanner Removal", messageKey: "ups_container_scannerremoved", overlay: this._overlay()});
				//player.ship.bounty += missionVariables.ups_galcop_punishment;
                player.ship.setBounty(player.ship.bounty + missionVariables.ups_galcop_punishment, "ups_scannerremoved");
			}
			else if (player.ship.equipmentStatus("EQ_UPS_IDCLEANER") === "EQUIPMENT_OK" && Math.random() < 0.2)
			{
				player.ship.removeEquipment("EQ_UPS_IDCLEANER");
				missionVariables.ups_galcop_punishment = 15;
				mission.runScreen({screenID: "ups-container", title: "ID Removal", messageKey: "ups_container_idcleanerremoved", overlay: this._overlay()});
				//player.ship.bounty += missionVariables.ups_galcop_punishment;
                player.ship.setBounty(player.ship.bounty + missionVariables.ups_galcop_punishment, "ups_idcleanerremoved");
			}
		}
    }
    else if (player.ship.dockedStation.name === "Pirate Rock")
    {
        var containerOffer = this.offering;
        if (missionVariables.ups_ecount > 1 && (containerStatus === "LOGGING" || containerStatus === "LOGGING2" || containerStatus === "SEARCHING3" || containerStatus === "SEARCHING4"))
        {
            mission.runScreen({screenID: "ups-container", title: "Pirate Offer", messageKey: "ups_container_pirateoffer",
                                choicesKey: "ups_container_pirateoffer_yesno"}, this.choiceEvaluation);
            this.offering = "UPS_CONTAINER_PIRATE";
        }
        if (containerOffer === "UPS_CONTAINER_PIRATE1")
        {
            mission.runScreen({screenID: "ups-container", title: "Pirate Offer", messageKey: "ups_container_pirateoffer2"});
            this.offering = "UPS_CONTAINER_PIRATE2";
        }
        if (containerOffer === "UPS_CONTAINER_PIRATE2")
        {
            mission.runScreen({screenID: "ups-container", title: "Pirate Offer", messageKey: "ups_container_pirateoffer2a"});
            this.offering = "UPS_CONTAINER_PIRATE3";
        }
        if (containerOffer === "UPS_CONTAINER_PIRATE3")
        {
            mission.runScreen({screenID: "ups-container", title: "Pirate Offer", messageKey: "ups_container_pirateoffer2b"});
            delete this.offering;
        }
    }
}

this.choiceEvaluation = function (choice)
{
    var containerOffer = this.offering;
    delete this.offering; // delete here in case a choice creates a new offer.
    if (this.mark)
    {
        mission.unmarkSystem({
                   system: this.ups_cplanet,
                   name: "ups_marker"
        })
    }
    switch (containerOffer)
    {
        case "UPS_CONTAINER1":
        {
            this.mark = false
            if (choice === "YESCon")
            {
                mission.setInstructionsKey("ups_containers_small1b", "ups_container");
                this.upsMarkSystem(this.ups_cplanet, this.name);
                this.ups_container = "SEARCHING";
                this.ups_timerstart = clock.days;
                missionVariables.ups_closecontact = "NO";
                delete this.ups_distance;
            }
            else if (choice === "LOOKCon")
            {
                this.upsShowDestination(this.ups_cplanet);
            }
            else if (choice === "NOCon")
            {
                delete this.ups_distance;
                this.ups_container = "NOT_NOW";
            }
            break;
        }
        case "UPS_CONTAINER2":
        {
            this.mark = false;
            if (choice === "YESCon")
            {
                mission.setInstructionsKey("ups_containers_small2", "ups_container");
                this.upsMarkSystem(this.ups_cplanet, this.name);
                this.ups_container = "SEARCHING2";
                }
            else if (choice === "LOOKCon")
            {
                this.upsShowDestination(this.ups_cplanet);
            }
            else if (choice === "NOCon")
            {
                this.ups_container = "NOT_NOW";
            }
            break;
        }
        case "UPS_CONTAINER3":
        {
            this.mark = false;
            if (choice === "YESCon")
            {
                mission.setInstructionsKey("ups_containers_small4", "ups_container");
                this.upsMarkSystem(this.ups_cplanet, this.name);
                this.ups_container = "SEARCHING3";
            }
            else if (choice === "LOOKCon")
            {
                this.upsShowDestination(this.ups_cplanet);
            }
            else if (choice === "NOCon")
            {
                this.ups_container = "NOT_NOW";
            }
            break;
        }
        case "UPS_CONTAINER_PIRATE":
        {
            if (choice === "YESCon")
            {
                player.credits += 5000;
                this.offering = "UPS_CONTAINER_PIRATE1";
                this.ups_container = "JOINED_PIRATES";
                missionVariables.TL_FOR_EQ_UPS_POLICE_SCANNER = 3;
                missionVariables.TL_FOR_EQ_UPS_IDCLEANER = 5;
            }
            else if (choice === "NOCon")
            {
                player.ship.launch();
                player.commsMessage("Yes, destroy the pirates.", 5);
                this.disableCloakingDevice();
            }
            break;
        }
        default: log(this.name, "Encountered an illegal mission offer: "+this.offering);
    }
}

this.upsShowDestination = function (ID)
{
    if (!this._savedTargetSet) this._savedTarget = player.ship.targetSystem;
    this._savedTargetSet = true;
    
    this.mark = true; // Keep this selected system.
    if (player.ship.equipmentStatus("EQ_ADVANCED_NAVIGATIONAL_ARRAY") == "EQUIPMENT_OK")
    {
        player.ship.targetSystem = ID;
        mission.runScreen({title: "Long Range Chart", screenID:"ups-map", backgroundSpecial: "LONG_RANGE_CHART_SHORTEST"});
    }
    else
    {
        mission.markSystem({
                   system: ID,
                   name: "ups_marker",
                   markerColor: "cyanColor",
                   markerScale: 2.0,
                   markerShape: "MARKER_DIAMOND"
        })
        var message = expandMissionText("ups_long_range_screen", {ups_system: System.systemNameForID(ID)});
        mission.runScreen({title: "Long Range Chart", screenID:"ups-map", message: message, backgroundSpecial: "LONG_RANGE_CHART"});
    }
}

this.shipWillEnterWitchspace = function ()
{
    this.stopScanners();
}

this.shipExitedWitchspace = function ()
{
    if (system.isInterstellarSpace)
    {
        if (player.credits > 10000 && missionVariables.ups_tcount > 2 && player.ship.fuel < 3 && Math.random() < 0.5)
        {
            if (worldScripts.transportSchedule || worldScripts.transportschedule !== undefined)  // has Transports.OXP installed
                system.addShipsToRoute("ups-womafueltanker", 1, 0.2, 'wp')
            else
                system.addShipsToRoute("ups-fueltanker", 1, 0.2, 'wp');
        }
        // this.startScanners();  // no scanning in interstellar space? One can still do it with the primed key.
        return;
    }
    this.setUpShips();
	var containerStatus = this.ups_container;
    if (containerStatus === "NOT_NOW" && Math.random() < 0.11 && !missionVariables.ups_target) this.ups_container = "NO";
    if (containerStatus === "NO")
    {
        if (system.government > 3 && Math.random() < 0.05)
        {
            if (this.ups_ccount < 10 || Math.random() < 0.25)
                this.ups_container = "BOA";
        }
        else if (this.ups_ccount > 1 && system.government < 2 && Math.random() < 0.15 && !system.sun.hasGoneNova)
        {
            if (this.ups_c1count > 10 && Math.random() > 0.33)
            {
                if (Math.random() > 0.50) this.ups_container = "SPILLAGE";
            }
            else
            {
                this.ups_container = "LOGGING";
                this.ups_cplanet = system.ID;
                missionVariables.ups_planetname = system.name;
                this.ups_timerstart = clock.days;
            }
        }
        else if (this.ups_c1count > 1 && system.government === 0 && Math.random() < 0.2 && !system.sun.hasGoneNova)
        {
            if (this.ups_c2count < 10 || Math.random() < 0.33)
            {
                this.ups_container = "LOGGING2";
                this.ups_cplanet = system.ID;
                missionVariables.ups_planetname = system.name;
                this.ups_timerstart = clock.days;
            }
        }
        else if (player.credits > 10000 && system.techLevel < 6 && Math.random() < 0.04 && missionVariables.ups_tcount > 2 && !system.sun.isGoingNova)
        {
            player.ship.fuelLeakRate = 20;
            player.consoleMessage(expandDescription("[danger-fuel-leak]"), 4);
            if (worldScripts.transportSchedule)  // has Transports.OXP installed
                system.addShips("ups-womafueltanker", 1); else system.addShips("ups-fueltanker", 1);
            system.addShipsToRoute("sunskim-trader", 2, Math.random(), "sw");
            system.addShipsToRoute("sunskim-trader", 2, Math.random(), "sw");
            system.addShipsToRoute("ups-sunskim", 2, Math.random(), "sw");
            system.addShipsToRoute("ups-sunskim", 2, Math.random(), "sw");
            system.addShipsToRoute("ups-sunskim", 2, Math.random(), "sw");
        }
    }
    else
    {
		if (containerStatus.indexOf("SEARCHING") === 0) // one of the 4 searching modi.
        {
            if (player.score > 1000)
            {
                system.addShipsToRoute(this.pirate, 4, Math.random(), "wp");
                system.addShipsToRoute(this.tiger, 4, Math.random(), "wp");
                system.addShipsToRoute("trader", 1, Math.random(), "wp");
            }
        }
        if (containerStatus.indexOf("JOINED_PIRATES") === 0) // one of the 2 joining modi.
        {
            system.addShipsToRoute("trader", (8 - system.economy), 0.25, "wp");
            if (system.economy < 4) system.addShipsToRoute("upstrader", 1);
            if (system.economy < 2)
            {
                system.addShipsToRoute("upstrader", 2, Math.random(), "wp");
                system.addShipsToRoute("upstrader", 1, Math.random(), "wp");
            }
            if (system.government > 4)
            {
                system.addShipsToRoute("police", 3, Math.random(), "wp");
                system.addShipsToRoute("police", 3);
            }
        }
        if (containerStatus === "SPILLAGE3")
        {
            if (!this.ups_container_count)
            {
                this.ups_container = "NOT_NOW";
                if (this.ups_cplanet)
                {
                    this.upsUnmarkSystem(this.ups_cplanet, this.name);
                    mission.setInstructionsKey(null);
                    delete this.ups_cplanet;
                    delete this.ups_container_award;
                    delete this.ups_container_count;
                    delete this.ups_total_container_count;
					delete this.ups_container_remaining;
                }
            }
        }
        if (missionVariables.ups_target)
        {
            var pasedTime = clock.days - this.ups_timerstart;
            if (pasedTime > 15)
			{
				delete missionVariables.ups_target;
                delete this.ups_timerstart;
				this.ups_container = "NO";
			}
            else
            {
            var happyTraders = system.shipsWithPrimaryRole("trader", player.ship);
            for (var i=0; i<happyTraders.length;i++)
                if (happyTraders[i].isValid) //could have already been destroyed or not present
                {
                    if (happyTraders[i].AI === "route1traderAI.plist" && pasedTime < 15 * Math.random())
						happyTraders[i].switchAI("upsHailingTraderAI.plist");
                }
            }
        }
    }
    if (player.ship.equipmentStatus("EQ_UPS_IDCLEANER") === "EQUIPMENT_OK")
    {
        if (!missionVariables.ups_status_backup && system.government === 0)
        {
            missionVariables.ups_status_backup = player.ship.bounty;
            player.ship.bounty = 0;
        }
        if (missionVariables.ups_status_backup && missionVariables.ups_status_backup > -1 && system.government > 0)
        {
            player.ship.bounty += missionVariables.ups_status_backup;
            delete missionVariables.ups_status_backup;
        }
    }
    if (system.government > 3 && this.ups_c1count > 2 && Math.random() < 0.04)
    {
        system.addShipsToRoute("derelict_ptt", 1, Math.random(), "wp");
    }
    this.startScanners();
}

this.playerEnteredNewGalaxy = function (newGalaxyNumber)
{
    //reset missions on switching galaxy
    this.ups_container = "NOT_NOW";
    delete this.ups_cplanet;
	delete missionVariables.ups_cplanetname;
    delete this.ups_container_count;
	delete this.ups_container_award;
    delete this.ups_total_container_count;
	delete missionVariables.ups_planet;
	delete missionVariables.ups_planetname; // variable only used for missiontext & choices.
	delete this.ups_timerstart;
    mission.setInstructionsKey(null);
    delete missionVariables.TL_FOR_EQ_UPS_POLICE_SCANNER;
    delete missionVariables.TL_FOR_EQ_UPS_IDCLEANER;
}

this.setUpShips = function ()
{
    var containerStatus = this.ups_container;
	if (this.ups_cloaking) this.enableCloakingDevice();
    if (containerStatus === "SEARCHING" && this.ups_cplanet === system.ID  && missionVariables.ups_closecontact !== "DOCKED")
    {
        if (system.countShipsWithRole("boa_ptt") === 0)
        {
            system.addShipsToRoute(this.boa(), 1, Math.random() * 0.5 + 0.2, "wp");
        }
        missionVariables.ups_closecontact = "NO";
    }
    if (containerStatus === "SEARCHING2" && this.ups_cplanet === system.ID)
    {
        if (system.countShipsWithRole("python_ptt") === 0)
        {
            system.addShipsToRoute(this.python(), 1, Math.random(), "pw");
        }
    }
    if (containerStatus === "SEARCHING3")
    {
        if (this.ups_cplanet === system.ID)
        {
            if (system.countShipsWithRole("upsbase") === 0)
            {
                this.addPirateRock("pirate-upsbase", false);
            };
			this.disableCloakingDevice();
        }
    }
    if (containerStatus.indexOf("JOINED_PIRATES") === 0)
    {
        if ((system.government === 0 && system.pseudoRandom100 < 5) || this.ups_cplanet === system.ID)
        {
            if (system.countShipsWithRole("upsbase") === 0)
            {
                this.addPirateRock("pirate-upsbase", false);
            };
			this.disableCloakingDevice();
        }

    }
    if (containerStatus === "SEARCHING4" && this.ups_cplanet === system.ID)
    {
        if (system.countShipsWithRole("upsbase") === 0)
        {
            this.addPirateRock("pirate-upsbase2", true);
            this.ups_timerstart = clock.days;
        };
		this.disableCloakingDevice();
    }
    if (missionVariables.ups_private_escorts)
    {
        if ((clock.days - missionVariables.ups_private_escorts_date) < 11)
        {
            if (system.countShipsWithRole("ups_private_escort") === 0)
            {
                system.addGroup("ups_private_escort", missionVariables.ups_private_escorts, player.ship.position, 15000);
            }
        }
        else
        {
            var myEscorts = system.shipsWithRole("ups_private_escort");
            for (var i=0; i < myEscorts.length; i++)
            {
                if (myEscorts[i].isValid) myEscorts[i].AIState = "CANCEL_CONTRACT";
            }
            delete missionVariables.ups_private_escorts;
            delete missionVariables.ups_private_escorts_date;
            player.consoleMessage(expandDescription("[ups_escort_expired]"), 6);
            player.ship.removeEquipment("EQ_UPS_PRIVATE_ESCORT");  // Code for comatibility with old versions.
        }
    }
}

this.addPirateRock = function (rock, strong)
{
    var baseposition = Math.random()*0.2+0.3;
    var position1 = Vector3D(0, 0, baseposition).fromCoordinateSystem("wpu");
    var position2 = Vector3D(0, 0, baseposition - 0.15).fromCoordinateSystem("wpu");
    var rock = system.addShips(rock, 1, position1)[0];
    system.addShips("asteroid", 8, rock.position, 15E3);
    system.addShips("police", 2, position2);
    system.addShips(this.pirate, 2, position2);
    if (strong)
    {
        system.addShips("upsdefender", 8, rock.position, 15E3);
        system.addShips(this.tiger, this.ups_c3count, rock.position, 15E3);
    }
    rock.orientation = [1,0,0,0]; // entrance towards planet.
}

this.enableCloakingDevice = function ()
{
    if (this.ups_cloaking && this.ups_cloaking === "CRIPPLED")
    {
        var damaged = (player.ship.equipmentStatus("EQ_CRIPPLED_CLOAKING") === "EQUIPMENT_DAMAGED"); // even the cripled one can get damaged in fight.

        if (player.ship.equipmentStatus("EQ_CRIPPLED_CLOAKING") !== "EQUIPMENT_UNAVAILABLE") // player has the equipment
		{
			player.ship.removeEquipment("EQ_CRIPPLED_CLOAKING");
			player.ship.awardEquipment("EQ_CLOAKING_DEVICE");
            if (damaged) player.ship.setEquipmentStatus("EQ_CLOAKING_DEVICE", "EQUIPMENT_DAMAGED");
		}

        delete this.ups_cloaking;
    };
}

this.disableCloakingDevice = function ()
{
	if (player.ship.equipmentStatus("EQ_CLOAKING_DEVICE") === "EQUIPMENT_OK")
	{
		player.consoleMessage("Cloaking jammed", 3);
		player.ship.removeEquipment("EQ_CLOAKING_DEVICE");
		player.ship.awardEquipment("EQ_CRIPPLED_CLOAKING");
		this.ups_cloaking = "CRIPPLED";
	}
}

this.playerBoughtEquipment = function (equipment)
{
	if (equipment === "EQ_UPS_PRIVATE_ESCORT")
	{
			missionVariables.ups_private_escorts = 8;
			missionVariables.ups_private_escorts_date = clock.days;
			// player.ship.removeEquipment("EQ_UPS_PRIVATE_ESCORT"); // is now invisible equipment.
	}
	if (equipment === "EQ_UPS_IDCLEANER")
	{
		missionVariables.ups_status_backup = player.ship.bounty;
		player.ship.bounty = 0;
	}
	if (equipment === "EQ_UPS_UNIVERSAL_SCANNER_REMOVER")
	{
		var refund = EquipmentInfo.infoForKey("EQ_UPS_UNIVERSAL_SCANNER").price/10 * 0.75;
        if (player.ship.equipmentStatus("EQ_UPS_UNIVERSAL_SCANNER") === "EQUIPMENT_DAMAGED") refund /= 10;
        player.ship.removeEquipment("EQ_UPS_UNIVERSAL_SCANNER");
		player.ship.removeEquipment("EQ_UPS_UNIVERSAL_SCANNER_REMOVER");
		player.credits += refund;
	}
}

this.boa = function ()
{
	// all variations have the common role "boa_ptt".
	var boa;
    var count = Math.floor(this.ups_ccount / 4);
    switch (count)
    {
        case 0: boa = "boa_ptt_easy"; break;
        case 1: boa = "boa_ptt_medium"; break;
        case 2: boa = "boa_ptt_hard"; break;
        default: boa = "boa_ptt_very_hard";
    }
	return boa;
}

this.python = function ()
{
	// all variations have the common role "python_ptt".
	var python
    var count = Math.floor(this.ups_ccount / 5);
    switch (count)
    {
        case 0: python = "python_ptt_easy"; break;
        case 1: python = "python_ptt_medium"; break;
        default: python = "python_ptt_hard";
    }
	return python;
}

this.spillCargo = function ()
{
    for (var i=0; i < 4; i++)
    {
        var scavenger = system.addShipsToRoute("scavenger", 1)[0];
        if (scavenger && scavenger.isValid) scavenger.switchAI("route1UpsScavengerAI.plist");
        system.addShipsToRoute("ups-spilled-container", 5);
        system.addShipsToRoute("cargopod", 5);
    }
    this.ups_container_count = 0;
    this.ups_total_container_count = 0;
    this.ups_container_remaining = 20;
 }

this.ups_scanners = function ups_scnners()
{
    if (player.ship.status !== "STATUS_IN_FLIGHT")
    {
        this.scannertimer.stop();
        return;
    }
    if (player.ship.equipmentStatus("EQ_UPS_POLICE_SCANNER") === "EQUIPMENT_OK")
    {
        this.upsPoliceScanner();
    }
    else if (player.ship.equipmentStatus("EQ_UPS_UNIVERSAL_SCANNER") === "EQUIPMENT_OK")
    {
        this.upsUniversalScanner();
    }
}

this.upsPoliceScanner = function()
{
    // function is also called from equipment script.
    player.consoleMessage("scanned: "
        + system.countEntitiesWithScanClass("CLASS_POLICE") +" GalCop ships, "
        + system.countShipsWithPrimaryRole("hunter") + " hunters "
        + system.countShipsWithPrimaryRole("trader") + " traders and "
        + system.countShipsWithPrimaryRole("escort") + " escorts.", 6
    );
    if (system.government > 0 && Math.random() < 0.06)
    {
        //player.ship.bounty++;
        player.ship.setBounty(player.ship.bounty + 1, "ups_policescanneruse");
        system.addShipsToRoute("police", 1);
        system.addShipsToRoute("police", 1, Math.random(), "wp");
        if (system.government > 5) system.addShips("police", 1);
    }
}

this.upsUniversalScanner = function()
{
    // function is also called from equipment script.
    player.consoleMessage("Scanned: "
        + system.countShipsWithPrimaryRole("pirate") + " pir, "
        + system.countEntitiesWithScanClass("CLASS_POLICE") + " pol, "
        + system.countShipsWithPrimaryRole("hunter") + " hunt, "
        + system.countShipsWithPrimaryRole("trader") + " trad, "
        + system.countShipsWithPrimaryRole("escort") + " esc, "
        + system.countEntitiesWithScanClass("CLASS_THARGOID") + " tharg.", 6
    );
}

this.positionToDirection = function(pos)
{
    var pos1 = system.info.coordinates;
    var pos2 = new Vector3D(pos);
    var dx, dy, x2, y2;

    dx = pos2.x - pos1.x;
    dy = pos2.y - pos1.y;
    x2 = Math.abs(dx);
    y2 = Math.abs(dy);

    if (x2 > 2.41421 * y2)
    {  // x2 > y2 / Math.tan(Math.PI/8)
        return dx > 0? "east" : "west";
    }
    if (y2 > 1.732 * x2)
    {
        return dy < 0? "north" : "south";
    }
    if (dx > 0)
    {
        return dy < 0? "north-east" : "south-east";
    }
    else
    {
        return dy < 0? "north-west" : "south-west";
    }
};

this.startScanners = function ()
{
    if (player.ship.equipmentStatus("EQ_UPS_UNIVERSAL_SCANNER") === "EQUIPMENT_OK" || player.ship.equipmentStatus("EQ_UPS_POLICE_SCANNER") === "EQUIPMENT_OK")
    {
        if (this.scannertimer)
        {
            if (!this.scannertimer.isRunning) this.scannertimer.nextTime = clock.absoluteSeconds + 20;
            this.scannertimer.start();
        }
        else
        {
            this.scannertimer = new Timer(this, this.ups_scanners, 20, 60);
        }
    }
}

this.stopScanners = function ()
{
    if (this.scannertimer) this.scannertimer.stop();
}

this._reputation = function ()
{
    return player.parcelReputation;
}

this._overlay = function() {
    if (player.ship.hudHidden || player.ship.hudAllowsBigGui) {
        return {name:"upsnews-large.png", height:546};
    } else {
        return {name:"upsnews-small.png", height:546};
    }
}

this._getSystemsInRange = function(range) {
    var infoList = SystemInfo.systemsInRange(range);
    // remove any systems you can't reach
    for (var i = infoList.length - 1; i >= 0; i--) {
        var myRoute = System.infoForSystem(galaxyNumber, system.ID).routeToSystem(infoList[i], player.ship.routeMode);
        if (!myRoute || myRoute.route.length === 0) {
            infoList.splice(i, 1);
        }
        myRoute = null;
    }
    return infoList;
}