"use strict";

this.name        = "ups_docs";
this.author      = "eric walch";
this.copyright   = "2008 eric walch.";
this.license     = "CC BY-NC-SA 3.0";
this.description = "Docs missions of UPS Courier";

//1.9 asteroids are chosen from any asteroid available instead of using special scripted asteroids.

this.startUp = function ()
{
    this.pirate = "pirate";
    if (!missionVariables.ups_docs) this.$initialise();
    this.$loadVariables();
    this.mark = false;
	if (missionVariables.ups_galaxy != galaxyNumber)
	{
		log(this.name, "UPS detected the player is in an other galaxy than the last save with UPS installed. Now resetting missions.")
		var galaxy = missionVariables.ups_galaxy;
		if (++galaxy === 8) galaxy = 0;  // simulate jump in next galaxy since last save for proper resetting old missions.
        worldScripts["ups_docs"].playerEnteredNewGalaxy(galaxy);
        worldScripts["ups_parcel"].playerEnteredNewGalaxy(galaxy);
        worldScripts["ups_container"].playerEnteredNewGalaxy(galaxy);
        worldScripts["ups_sun"].playerEnteredNewGalaxy(galaxy);
        worldScripts["ups_slaves"].playerEnteredNewGalaxy(galaxy);
        missionVariables.ups_galaxy = galaxyNumber;
	}
    if (!missionVariables.upsVersionUpdated)
    {
        // Uses bits to remember changes so this varriable can be used also for furure changes
        // bit 1: Markers are updated.
        this.updateUpsMarkers();
        missionVariables.upsVersionUpdated = "1";
    }
    this.cleanUpMarkers();
}

this.$initialise = function ()
{   // setting up all mission variables.
	missionVariables.ups_docs = "NO";
	missionVariables.ups_parcel = "NO";
	missionVariables.ups_container = "NO";
    missionVariables.ups_slaves = "NO";
    var initialitations = ["dcount", "pcount", "ccount", "c1count", "c2count", "ecount", "galcop_punishment", "reputation"];
    for (var i=0; i<initialitations.length; i++) missionVariables["ups_"+initialitations[i]] = 0;
	missionVariables.ups_galaxy = galaxyNumber;  // needed to detect jumps with ups de-installed.
    // galaxyNumber is GLOBAL and may change to player.galaxyNumber  in future!!!
}

this.vars = {
    // table used for translation between missionVariables and worldScript variables. MissionVarriables are slow, so
    // the most frequent used ones are maintained as real varriables during game and only turned back into
    // missionVariables on saving.
    "ups_docs": ["ups_docs", "ups_dcount", "ups_dplanet", "ups_reputation"],
    "ups_parcel": ["ups_parcel", "ups_pcount", "ups_pplanet", "ups_ptimerstart"],
    "ups_container": ["ups_container", "ups_ccount", "ups_c1count", "ups_c2count", "ups_c3count", "ups_cplanet",
                      "ups_timerstart", "ups_container_count", "ups_container_award", "ups_container_message",
                      "ups_container_destroyed"],
    "ups_slaves": ["ups_slaves", "ups_slaverescue", "ups_slplanet", "ups_sl2planet", "ups_slcount", "ups_slavesrescued", "ups_sl2count", "ups_slavename"],
    "ups_sun": ["ups_sun", "ups_blackbox", "ups_sunvisits", "ups_sunbase", "ups_splanet", "ups_splanetname", "ups_stimerstart",
                "ups_rescue", "ups_s2count", "ups_s2award", "ups_stationparts"]
}

this.$loadVariables = function ()
{
    // Make sure the next variables always exist.
    var initialitations = ["dcount", "pcount", "ccount", "c1count", "c2count", "ecount", "galcop_punishment", "reputation", "galaxy"];
    for (var i=0; i<initialitations.length; i++)
    {
        if (!missionVariables["ups_"+initialitations[i]]) missionVariables["ups_"+initialitations[i]] = 0;
    }

	// missionvariables are very slow. Convert the most intensive used ones to local JS variables.
    // do it in the script with the initialisation code because we don' know if the other scripts will initialise after this one.
    var missions = Object.getOwnPropertyNames(this.vars);
    var i, j;
    for (j=0; j < missions.length; j++)
    {
        var m = missions[j];
        for (i=0; i < this.vars[m].length; i++)
        {
            worldScripts[m][this.vars[m][i]] = missionVariables[this.vars[m][i]];
        }
    }
    if (missionVariables.ups_difference) delete missionVariables.ups_difference; // no longer used.
}

this.playerWillSaveGame = function ()
{
    // update missionvariables to be stored.
    // I do it this way to stay compatible with old versions. For a new OXP it is
    // better to use the JSON function to convert all into a single mission variable.
    var missions = Object.getOwnPropertyNames(this.vars);
    var i, j;
    for (j=0; j < missions.length; j++)
    {
        var m = missions[j];
        for (i=0; i < this.vars[m].length; i++)
        {
            if (worldScripts[m][this.vars[m][i]])
            {
                // only create mission variables for existing variables
                missionVariables[this.vars[m][i]] = worldScripts[m][this.vars[m][i]];
            }
            else
            {
                // make sure we don't leave unused missionVariables from a loaded game.
                delete missionVariables[this.vars[m][i]];
            }
        }
    }
    this.cleanUpMarkers();
}

this.shipWillDockWithStation = function (station)
{
    if (station.isMainStation)
    {
        this.ups_reports(station);
    }
}

this.shipLaunchedFromStation = function ()
{
    this.setUpShips();
    if (this.ups_docs === "ASTEROID") this.setUpAsteroidMission();
}

this.missionScreenOpportunity = function ()
{
    if (player.ship.docked)
    {
        this.missionOffers();
    }
}

this.requestLongRangeChart = function (system, script)
{
    this.lookLongRangeChartScreen = true;
    this.askingScript = script;
    this.planetShowNumber = system;
}

this.longRangeChart = function longRangeChart() // routine used by 4 ups world scripts
{
    if (guiScreen === "GUI_SCREEN_LONG_RANGE_CHART")
    {
        if (this.mark) {this.upsUnmarkSystem(this.planetShowNumber, this.name); this.mark = false}
        else {this.upsMarkSystem(this.planetShowNumber, "ups_docs"); this.mark = true};
    }
    else
    {
        this.missionChartTimer.stop();
        this.upsUnmarkSystem(this.planetShowNumber, this.name);
        this.mark = true; // Keep this selected system.
    }
}

this.guiScreenChanged = function (newScreen, oldScreen)  // routine used by all 5 ups world scripts
{
    if (this.restartMissionOffer && newScreen !== "GUI_SCREEN_LONG_RANGE_CHART")
    {
        this.restartMissionOffer = false;
        if (guiScreen !== "GUI_SCREEN_MISSION") this.missionOffers();
    }
	if (!this.lookLongRangeChartScreen) return;
	if (!player.ship.docked) this.lookLongRangeChartScreen = null;
	if (guiScreen === "GUI_SCREEN_LONG_RANGE_CHART")
	{
		this.lookLongRangeChartScreen = null;
		if (this.missionChartTimer) this.missionChartTimer.start()
		else this.missionChartTimer = new Timer(this, this.longRangeChart, 0.5, 0.5);
        worldScripts[this.askingScript].restartMissionOffer = true;
	}
	else player.consoleMessage("Go to the long range screen: F6/F6", 10);
}

this.ups_reports = function (station)
{
    if (system.government === 6 && this.ups_docs === "SMALL_DELIVERY")
    {
        if (this.ups_dplanet !==  system.ID  && this.ups_dcount > 0) // The case (this.ups_dcount == 0) is handled in a missionscreen.
        {
            this.ups_docs = "NOT_NOW";
            player.credits += 100;
            player.addMessageToArrivalReport(expandMissionText("ups_docs_arrival_report"));
            mission.setInstructionsKey(null);
            this.ups_dcount++;
            missionVariables.ups_docs_appearance = null;
            this.increaseReputation(1);
        }
    }
    if (this.ups_docs === "EARTHQUAKE_DELIVERY" && this.ups_dplanet ===  system.ID)
    {
        if (missionVariables.ups_earthquake_cargo > 0 && manifest["machinery"] > 0)
        {
            var machinery = manifest["machinery"];
            if (machinery > missionVariables.ups_earthquake_cargo) machinery = missionVariables.ups_earthquake_cargo;
            player.addMessageToArrivalReport(expandMissionText("ups_earthquake_arrival_report",{machine_count: machinery}));
            manifest["machinery"] -= machinery;
            missionVariables.ups_earthquake_cargo -= machinery;
            this.machineDelivery = true;
        }
    }
};

this.addUpsContract = function ()
{
    var thisSystem = system.info;

    var targetSystems = SystemInfo.filteredSystems(this, function(other){
           return (other.systemID !== thisSystem.systemID) && other.government == 6 && !system.sun.hasGoneNova;
    });
    var count = Math.ceil(Math.random()*5);

    for (var i = 0; i<count; i++)
    {
        var contractSystem = targetSystems[Math.floor(Math.random() * targetSystems.length)];
        var contractRoute = system.info.routeToSystem(contractSystem);
		// if there is no route to the destination, skip this one
		if (contractRoute == null) continue;
        var contractDescription = expandDescription("[ups_condition] [ups_document_description]");
        contractDescription = contractDescription.charAt(0).toUpperCase() + contractDescription.slice(1);

        var parcel =
        {
            destination: contractSystem.systemID,
            sender: "UPS courier",
            deadline: clock.seconds + contractRoute.time * 3600 + 3600*24*2,
            payment: Math.floor(contractRoute.distance * (3+Math.random()) + (Math.pow(contractRoute.route.length,1+(0.3*player.parcelReputation))) + 50),
            description: contractDescription,
            route: contractRoute
        }

        worldScripts["oolite-contracts-parcels"]._addParcelToSystem(parcel);
    }
};

this.missionOffers = function ()
{
    if (this._savedTargetSet) player.ship.targetSystem = this._savedTarget
    this._savedTarget = null;
    this._savedTargetSet = false;

	var docsStatus = this.ups_docs;
    if (player.ship.dockedStation.isMainStation)
    {

        if (docsStatus === "YES"  && system.government === 6 && !worldScripts["ups_docs"].lookLongRangeChartScreen)
        {
            var chronicleSystem = new Array(124, 39, 36, 41, 198, 73, 149, 226)[galaxyNumber]; // define one per galaxy
			if (player.ship.bounty < 2 && this.ups_dcount > 20 &&
                    system.ID != chronicleSystem && (Math.random() < 0.2 || this.mark))
            {
                this.ups_dplanet = chronicleSystem;
				missionVariables.ups_dplanetname = System.systemNameForID(chronicleSystem);
				missionVariables.ups_docs_appearance = expandDescription("[ups_numbers] [ups_condition] %I");
				if (!missionVariables.ups_chronicle_content)
                {
					missionVariables.ups_chronicle_content = expandDescription("the [25] [27] on %H and the recent [8]");
                }
                var message = expandMissionText("ups_docs_chronicle_offer", {ups_inhabitants: system.inhabitantsDescription, dockedStationName: player.ship.dockedStation.displayName});
                mission.runScreen({screenID: "ups-docs", title: "Document Transport", message: message,
                                    overlay: this._overlay(), choicesKey: "ups_docs_accepted_yesnolook"},
                                    this.choiceEvaluation);
                this.offering = "UPS_DOCS_CHRONICLE";
            }
            else if (player.ship.bounty < 3)
            {
                missionVariables.ups_docs_appearance = expandDescription("[ups_numbers] [ups_condition] %I");
                this.today = new Date();
                if (this.today.getMonth()==11 && this.today.getDate() > 15 && this.today.getDate() < 26)
                    missionVariables.ups_docs_appearance += " Christmas";

                var key = (!this.ups_dcount) ? "ups_docs_1st_offer" : "ups_docs_offer";
                var message = expandMissionText(key, {dockedStationName: player.ship.dockedStation.displayName});
                mission.runScreen({screenID: "ups-docs", title: "Document Transport", message: message,
                                        overlay: this._overlay(), choicesKey: "ups_docs_accepted_yesno"},
                                        this.choiceEvaluation)
                this.offering = "UPS_DOCS";
            }
        }
        if (docsStatus === "SMALL_DELIVERY" && system.government === 6)
        {
            if (this.ups_dplanet ===  system.ID)
            {
                this.ups_docs = "ACCEPTED";
                mission.runScreen({screenID: "ups-docs", title: "Document Transport", messageKey: "ups_back_at_home_docs", overlay: this._overlay()});
                this.increaseReputation(-1);
            }
            else
            {
                if (!this.ups_dcount)
                {
                    this.ups_docs = "NOT_NOW";
                    player.credits += 100;
                    var message = expandMissionText("ups_docs_1st_unloading", {populationCount: system.population * 100});
                    mission.runScreen({screenID: "ups-docs", title: "Document Transport", message: message, overlay: this._overlay()})
                    mission.setInstructionsKey(null);
                    this.ups_dcount++;
                    missionVariables.ups_docs_appearance = null;
                    this.increaseReputation(1);
                }
                else
                {
                    // written in arrival report now.
                };
                if (!worldScripts.GNN && worldScripts.snoopers && worldScripts.snoopers.version >= 2.0) worldScripts.snoopers.insertNews({ID: this.name, Agency: 1, Message: expandDescription("[ups_snoopers_docs]")});
                if (worldScripts.GNN && worldScripts.GNN.version >= 1.0) worldScripts.GNN._insertNews({ID: this.name, Agency: 1, Message: expandDescription("[ups_snoopers_docs]")});
            }
        }
        if (docsStatus === "LOGGING" && system.government === 6)
        {
            if (this._reputation() > 3)
            {
                if (this.ups_dplanet != system.ID && !worldScripts.ups_docs.lookLongRangeChartScreen)
                {
                    missionVariables.ups_docs_appearance = expandDescription("[ups_numbers] secret %I");
                    var message = expandMissionText("ups_docs_offer2", {dockedStationName: player.ship.dockedStation.displayName});
                    mission.runScreen({screenID: "ups-docs", title: "Document Transport", message: message,
                                        overlay: this._overlay(), choicesKey: "ups_docs_accepted_yesnolook"},
                                        this.choiceEvaluation);
                    this.offering = "UPS_DOCS2";
                }
                else if (this._reputation() < 0)
                {
                   this.ups_docs = "NOT_NOW";
                }
            }
        }
        if (docsStatus === "SPECIAL_DELIVERY")
        {
            if (this.ups_dplanet === system.ID && !missionVariables.ups_docbriefing)
            {
                missionVariables.ups_docbriefing = "YES";
                mission.runScreen({screenID: "ups-docs", title: "Document Transport", messageKey: "ups_docs_unloading2a", overlay: this._overlay()});
                mission.setInstructionsKey("ups_docs_small3");
            }
        }
        if (docsStatus === "CHRONICLE_DELIVERY")
        {
            if (this.ups_dplanet === system.ID)
			{
                this.ups_docs = "NOT_NOW";
				if (clock.hours/24 <= missionVariables.ups_dtimer)
				{
                    var message = expandMissionText("ups_docs_chronicle_unloading", {dockedStationName: player.ship.dockedStation.displayName});
                    mission.runScreen({screenID: "ups-docs", title: "Document Transport", message: message, overlay: this._overlay()});
                    this.ups_dcount++;
                    player.credits += 3000;
                    this.increaseReputation(4);
                    if (!worldScripts.GNN && worldScripts.snoopers && worldScripts.snoopers.version >= 2.0) worldScripts.snoopers.insertNews({ID: this.name, Agency: 1, Message: expandDescription("[ups_snoopers_docs]")});
                    if (worldScripts.GNN && worldScripts.GNN.version >= 1.0) worldScripts.GNN._insertNews({ID: this.name, Agency: 1, Message: expandDescription("[ups_snoopers_docs]")});
				}
				else
				{
                    mission.runScreen({screenID: "ups-docs", title: "Document Transport", messageKey: "ups_docs_chronicle_unloading_late", overlay: this._overlay()});
                    this.increaseReputation(-3);
				}
				this.upsUnmarkSystem(this.ups_dplanet, this.name);
				missionVariables.ups_docs_appearance = null;
                this.ups_dplanet = null
				mission.setInstructionsKey(null);
				missionVariables.ups_dtimer = null;
				missionVariables.ups_docs_type = null;
				missionVariables.ups_chronicle_content = null;
			}
			else
			{
                this.updateChronicleMessages();
				if (clock.hours/24 > missionVariables.ups_dtimer + 5)
				{
					this.upsUnmarkSystem(this.ups_dplanet, this.name);
					this.ups_docs = "NOT_NOW";
					mission.runScreen({screenID: "ups-docs", title: "Document Transport", messageKey: "ups_docs_chronicle_expired", overlay: this._overlay()});
					missionVariables.ups_docs_appearance = null;
                    this.ups_dplanet = null;
					missionVariables.ups_dtimer = null;
					missionVariables.ups_chronicle_content = null;
					mission.setInstructionsKey(null);
					this.increaseReputation(-5);
				}
			}
        }
        if (docsStatus === "ASTEROID_OFFER")
        {
            if (system.countShipsWithRole("asteroid") < 25) 
            {
                mission.runScreen({screenID: "ups-docs", title: "Asteroid Mission", messageKey: "ups_docs_asteroid1",
                                    overlay: this._overlay(), choicesKey: "ups_docs_asteroid_yesno"},
                                    this.choiceEvaluation);
                this.offering = "UPS_DOCS_ASTEROID";
            }
        }
        if (player.trumbleCount > 0 && (docsStatus === "NOT_NOW" || docsStatus === "NO"))
        {
            if (Math.random() < player.trumbleCount/20 && player.credits > 2000)
            {
                var message = expandMissionText("ups_trumble_repellant", {ups_trumble_count: player.trumbleCount, ups_trumble_plural: (player.trumbleCount === 1 ? "" : "s")});
                mission.runScreen({screenID: "ups-docs", title: "Document Transport", message: message,
                                    overlay: "trumblebox.png", choicesKey: "ups_trumble_repellant_yesno"},
                                    this.choiceEvaluation);
                this.offering = "UPS_DOCS_REPELLANT";
            }
        }
        if (docsStatus === "TRUMBLE_REPELLANT" && clock.days > missionVariables.ups_dtimer + 8)
        {
            this.ups_docs = "NOT_NOW";
            if (Math.random() < 0.5) mission.runScreen({screenID: "ups-docs", title: "Document Transport", messageKey: "ups_trumble_repellant_warning",
                overlay: this._overlay()});
            missionVariables.ups_dtimer = null;
        }
        if (docsStatus === "EARTHQUAKE_OFFER" && system.government === 6 && !worldScripts["ups_docs"].lookLongRangeChartScreen)
        {
            if (!this.mark)
            {
                var infoList = worldScripts.ups_container._getSystemsInRange(25);
                var targetSystem = infoList[Math.floor(Math.random()*infoList.length)];
                this.ups_dplanet = targetSystem.systemID;
                missionVariables.ups_dplanetname = targetSystem.name;
                missionVariables.ups_dplanet_inhabitants = targetSystem.inhabitants;
                targetSystem.description = expandDescription("The planet "+ targetSystem.name
                        + " is frequently plagued by earthquakes. The last big one however, largely destroyed the planetary production capacity.");
                targetSystem.productivity = targetSystem.productivity / 100;
                // now contralled through planetinfo.plist and upsMarket_earthquake.js
                //targetSystem.market = "ups_earthquake";
                this.mark = true;
            }
            missionVariables.ups_earthquake_cargo = player.ship.cargoSpaceCapacity * 3;
            missionVariables.ups_earthquake_reward = missionVariables.ups_earthquake_cargo * 70;
            mission.runScreen({screenID: "ups-docs", title: "Machinery Transport", messageKey: "ups_earthquake_offer",
                                choicesKey: "ups_earthquake_yesnolook", overlay: this._overlay()},
                                this.choiceEvaluation);
            this.offering = "UPS_DOCS_EARTHQUAKE_OFFER";
        }
        if (this.ups_docs === "EARTHQUAKE_DELIVERY" && this.machineDelivery)
        {
            if (missionVariables.ups_earthquake_cargo > 0)
            {
                mission.runScreen({screenID: "ups-docs", title: "Machinery Transport", messageKey: "ups_earthquake_delivery",
                                    overlay: this._overlay()});
                mission.setInstructionsKey("ups_docs_earthquake");
                this.launchEarthquakeRescue(3);
            }
            else
            {
                this.ups_docs = "NOT_NOW";
                mission.runScreen({screenID: "ups-docs", title: "Machinery Transport", messageKey: "ups_earthquake_final",
                                    overlay: this._overlay()});
                mission.setInstructionsKey("ups_docs_earthquake");
                this.upsUnmarkSystem(this.ups_dplanet, this.name);
                // an undefined marget throws an exception in 1.74, so explicit check for existence
                if (system.info.hasOwnProperty("market")) system.info.market = null;
                system.info.description = null;
                system.info.productivity = null;
                player.credits += missionVariables.ups_earthquake_reward;
                missionVariables.ups_earthquake_reward = null;
                missionVariables.ups_earthquake_cargo = null;
                mission.setInstructionsKey(null);
                this.increaseReputation(5);
            }
            this.machineDelivery = false;
        }
    }
	else if (player.ship.dockedStation.name.indexOf("Sun Research Station") === 0) // docked on one of the 4 research stations
    {
        if (docsStatus === "SPECIAL_DELIVERY")
        {
            this.ups_docs = "NOT_NOW";
            player.credits += 250;
            mission.runScreen({screenID: "ups-docs", title: "Document Transport", messageKey: "ups_docs_unloading2", overlay: this._overlay()});
            mission.setInstructionsKey(null);
            this.ups_dcount++;
            this.upsUnmarkSystem(this.ups_dplanet, this.name);
            this.increaseReputation(3);
            if (player.ship.dockedStation.name === "Sun Research Station Alpha" && !worldScripts["ups_sun"].ups_sun)
            {
                worldScripts["ups_sun"].initSunMission(missionVariables.ups_dplanetname, this.ups_dplanet);
            }
            missionVariables.ups_dplanetname = null;
            missionVariables.ups_docbriefing = null;
			missionVariables.ups_docs_type = null;
            missionVariables.ups_docs_appearance = null;
            if (!worldScripts.GNN && worldScripts.snoopers && worldScripts.snoopers.version >= 2.0) worldScripts.snoopers.insertNews({ID: this.name, Agency: 1, Message: expandDescription("[ups_snoopers_docs]")});
            if (worldScripts.GNN && worldScripts.GNN.version >= 1.0) worldScripts.GNN.insertNews({ID: this.name, Agency: 1, Message: expandDescription("[ups_snoopers_docs]")});
        }
    }
}

this.choiceEvaluation = function (choice)
{
    var docOffer = this.offering;
    delete this.offering;
    switch (docOffer)
    {
        case "UPS_DOCS":
        {
            if (choice === "YESDoc")
            {
                mission.setInstructionsKey("ups_docs_small");
                this.ups_docs = "ACCEPTED";
                this.ups_dplanet = system.ID;
            }
            else
            {
                if (choice === "NODoc")
                {
                    this.ups_docs = "NOT_NOW";
                    missionVariables.ups_docs_appearance = null;
                    this.increaseReputation(-2);
                }
            }
            break;
        }
        case "UPS_DOCS2":
        {
            this.mark = false;
            if (choice === "YESDoc")
            {
                mission.setInstructionsKey("ups_docs_small2");
                this.ups_docs = "SPECIAL_DELIVERY";
                this.upsMarkSystem(this.ups_dplanet, this.name);
                missionVariables.ups_docs_type = "secret documents";
            }
            else if (choice === "LOOKDoc")
            {
                this.upsShowDestination(this.ups_dplanet);
            }
            else if (choice === "NODoc")
            {
                this.ups_docs = "NOT_NOW";
            }
            break;
        }
        case "UPS_DOCS_CHRONICLE":
        {
            this.mark = false;
            if (choice === "YESDoc")
            {
                this.ups_docs = "CHRONICLE_DELIVERY";
                this.upsMarkSystem(this.ups_dplanet, this.name);
                missionVariables.ups_dtimer = clock.hours/24 + 15;
                missionVariables.ups_docs_type = "chronicle articles";
            }
            else if (choice === "LOOKDoc")
            {
                this.upsShowDestination(this.ups_dplanet);
            }
            else if (choice === "NODoc")
            {
                this.ups_docs = "NOT_NOW";
            }
            break;
        }
        case "UPS_DOCS_ASTEROID":
        {
            if (choice === "YESDoc")
            {
                this.offering === "UPS_DOCS_ASTEROID2";
                mission.runScreen({screenID: "ups-docs", title: "Asteroid Mission", messageKey: "ups_docs_asteroid2", model: "ups-splinter"});
                mission.setInstructionsKey("ups_docs_asteroid_short");
                this.ups_docs = "ASTEROID";
                missionVariables.ups_asteroid_count = 16;
            }
            else if (choice === "NODoc")
            {
                this.ups_docs = "NOT_NOW";
            }
            break;
        }
        case "UPS_DOCS_REPELLANT":
        {
            if (choice === "YESDoc")
            {
                player.credits -= 2000;
                this.ups_docs = "TRUMBLE_REPELLANT";
                missionVariables.ups_dtimer = clock.days;
            }
            else if (choice === "NODoc")
            {
                if (Math.random() < 0.3) this.trumbleRepellant = true;
            }
            break;
        }
        case "UPS_DOCS_EARTHQUAKE_OFFER":
        {
            this.mark = false;
            if (choice === "YESDoc")
            {
                this.ups_docs = "EARTHQUAKE_DELIVERY";
                this.upsMarkSystem(this.ups_dplanet, this.name);
                mission.setInstructionsKey("ups_docs_earthquake");
                missionVariables.ups_dplanet_inhabitants = missionVariables.ups_dplanet_inhabitants.toLowerCase();
            }
            else if (choice === "LOOKDoc")
            {
                this.upsShowDestination(this.ups_dplanet);
            }
            else if (choice === "NODoc")
            {
                this.ups_docs = "NOT_NOW";
            }
            break;
        }
        default: log(this.name, "Encountered an illegal mission offer: "+this.offering);
    }
}

this.upsShowDestination = function (ID)
{
    if (!this._savedTargetSet) this._savedTarget = player.ship.targetSystem;
    this._savedTargetSet = false;
    
    this.mark = true; // Keep this selected system.
    if (player.ship.equipmentStatus("EQ_ADVANCED_NAVIGATIONAL_ARRAY") == "EQUIPMENT_OK")
    {
        player.ship.targetSystem = ID;
        mission.runScreen({title: "Long Range Chart", screenID:"ups-map", backgroundSpecial: "LONG_RANGE_CHART_SHORTEST"});
    }
    else
    {
        mission.markSystem({
                   system: ID,
                   name: "ups_marker",
                   markerColor: "magentaColor",
                   markerScale: 2.0,
                   markerShape: "MARKER_DIAMOND"
        });
        var message = expandMissionText("ups_long_range_screen", {ups_system: System.systemNameForID(ID)});
        mission.runScreen({title: "Long Range Chart", screenID:"ups-map", message: message, backgroundSpecial: "LONG_RANGE_CHART"});
    }
}

this.shipExitedWitchspace = function ()
{
    if (system.isInterstellarSpace) return;
    this.setUpShips();
	var docStatus = this.ups_docs;
    if (docStatus === "NOT_NOW" && Math.random() < 0.11) this.ups_docs = "NO";
    if (docStatus === "ACCEPTED")
    {
        this.ups_docs = "SMALL_DELIVERY";
        system.addShipsToRoute("trader", 2, Math.random(), "pw");
    }
    if (docStatus === "NO")
    {
        if (Math.random() < 0.2 && system.government === 6)
        {
            if (this.ups_dcount > 30 && Math.random() < 0.2)
                this.ups_docs = "EARTHQUAKE_OFFER"
            else
                this.ups_docs = "YES";
        }
        else if (Math.random() < 0.05 && system.government === 6 && this.ups_dcount > 5 && !system.sun.hasGoneNova)
        {
            this.ups_docs = "LOGGING";
            this.ups_dplanet = system.ID;
            missionVariables.ups_dplanetname = system.name;
        }
        else if (Math.random() < 0.035 && this.ups_dcount > 20 && system.countShipsWithRole("asteroid") < 15)
        {
            this.ups_docs = "ASTEROID_OFFER";
        }
    }
    else
    {
        if (docStatus === "ASTEROID")
        {
            this.ups_docs = "NOT_NOW";
            missionVariables.ups_asteroid_count = null;
            mission.setInstructionsKey(null);
        }
        if (docStatus === "SMALL_DELIVERY")
        {
			system.addShipsToRoute("trader", 1);
			system.addShipsToRoute("trader", 2, Math.random(), "pw");
		};
		if (docStatus === "CHRONICLE_DELIVERY")
		{
			if (system.government < 3) this.addDocPirates(10 - 4 * system.government)
			else if (Math.random() < 0.1) this.addDocPirates(2);
            this.updateChronicleMessages();
		}
        if (docStatus === "SPECIAL_DELIVERY" && system.government < 2)
        {
			this.addDocPirates(10 - 5 * system.government);
        }
    }
    if (system.government === 6 && !system.sun.hasGoneNova && system.mainStation)
    {
        this.addUpsContract();
    }
}

this.updateChronicleMessages = function ()
{
    if (this.messagesTimer)
    {
        this.messagesTimer.start();
    }
    else
    {
        this.messagesTimer = new Timer(this, this.updateChronicleMessagesDelayed, 0, 2); // clock might not be valid yet.
    }
}

this.updateChronicleMessagesDelayed = function updateChronicleMessagesDelayed()
{
    if (missionVariables.ups_dtimer)
    {
        // we need a delayed clock readout after a jump or launch.
        var message;
        if (clock.isAdjusting)
        {
            message = expandMissionText("ups_docs_small4", {ups_days_left: "updating...."});
        }
        else
        {
            message = expandMissionText("ups_docs_small4", {ups_days_left: this.getTimeToGo(missionVariables.ups_dtimer)});
            this.messagesTimer.stop();
        }
        mission.setInstructions(message);
    }
    else
    {
        // mission has ended
        mission.setInstructionsKey(null);
        this.messagesTimer.stop();
    }
}

this.addDocPirates = function (count)
{
    system.addShips(this.pirate, count);
	system.addShipsToRoute(this.pirate, count, 0.15, "wp");
	system.addGroupToRoute("pirate", 5, Math.random(), "wp");
    var docpirates = system.shipsWithPrimaryRole("pirate");
    if (docpirates)
    {
        for (var i=0; i<docpirates.length; i++)
            if (docpirates[i].isValid && docpirates[i].AI === "pirateAI.plist")
            {
                this.changePirateAI(docpirates[i], "upsSecDocPirateAI.plist", this.sendUpsDocMessage);
                if (docpirates[i].bounty === 0) docpirates[i].bounty = Math.ceil(Math.random()*30+5);
            }
    }
	if (this.ups_dcount > 15 && count > 5)
	{
		system.addShipsToRoute("ups_pirate_leader", 1, 0.35, "pw");
		system.addShipsToRoute("ups_pirate_leader", 1, 0.85); // station route
		system.addShipsToRoute("ups_pirate_leader", 1, 0.75, "pw");
	}
}

this.changePirateAI = function (pirate, AI, message)
{
    if (pirate && pirate.isValid)
    {
        pirate.switchAI(AI);
        pirate.script.sendUpsMessage = message; // message is the function to attach that sends the messages.
    }
}

this.playerEnteredNewGalaxy = function ()
{
    // reset missions on galactic jump
    if (this.ups_docs != "SMALL_DELIVERY")
    {
        this.ups_docs = "NOT_NOW";
        mission.setInstructionsKey(null);
        var deletions = ["dplanetname", "docsbriefing", "dtimer", "docs_appearance", "docs_type", "chronicle_content"];
        for (var i=0; i<deletions.length; i++) delete missionVariables["ups_"+deletions[i]];
        delete this.ups_dplanet;
    }
    missionVariables.ups_galaxy = galaxyNumber;
}

this.setUpShips = function ()
{
	if (this.ups_docs === "SPECIAL_DELIVERY" && this.ups_dplanet === system.ID)
    {

        var dependancePos = Vector3D(0, 0, 2.6).fromCoordinateSystem("sps");
        var stationRole = (worldScripts["ups_sun"].ups_sun ? "ups_dependance2" : "ups_dependance");
        if (system.countShipsWithRole(stationRole) === 0)
        {
            system.addShips(stationRole, 1, dependancePos);
            system.addShips("ups-sunskim", 3, dependancePos);
            system.addShips("ups_shuttle", 8, dependancePos);
            worldScripts["ups_sun"].researcher_count = 0;
			worldScripts["ups_sun"].sunShips("ups-sunskim", 3, "sp");
			worldScripts["ups_sun"].sunShips(this.pirate, 2, "sp");
			worldScripts["ups_sun"].sunShips("ups-sun-police", 3, "sp");
			worldScripts["ups_sun"].sunShips("ups-sunskim", 4, "sw");
            if (worldScripts.transportSchedule)  // has Transports.OXP installed
            {
                worldScripts["ups_sun"].sunShips("fueltransport", 3, "sp");
            }
        }
    }
}

this.setUpAsteroidMission = function ()
{
    function locationAt (pos) {return basePosition.add(planetVector.multiply(pos));};
    var count = missionVariables.ups_asteroid_count;
    if (system.countShipsWithRole("ups-miner") === 0 && count > 8)
    {
        var basePosition = system.mainPlanet.position;
        var planetVector = Vector3D([0,0,1]); // unit vector pointing behind planet
        count = Math.ceil(count/2);

        system.addShips("asteroid", count, locationAt(180E3), 15E3);
        var specialAsteroids = system.addShips("asteroid", count, locationAt(225E3), 15E3);
        //add ups script to the spawned special asteroids. doing it this way allows asteroid storm scripts to run their spawn part that sets the color. does not work for the asteroid storm boulders though. I think we can live with it.
        for (var i = 0; i < specialAsteroids.length; i++) {
			specialAsteroids[i].setScript("upsAsteroid.js");
			//this is for the mission counter
            specialAsteroids[i].$upsAsteroid = true;
		}
        system.addShips("asteroid", count, locationAt(270E3), 15E3);
        specialAsteroids = system.addShips("asteroid", count, locationAt(315E3), 15E3);
        //add ups script to the spawned special asteroids. doing it this way allows asteroid storm scripts to run their spawn part that sets the color. does not work for the asteroid storm boulders though. I think we can live with it.
        for (i = 0; i < specialAsteroids.length; i++) {
			specialAsteroids[i].setScript("upsAsteroid.js");
			//this is for the mission counter
			specialAsteroids[i].$upsAsteroid = true;
		}
        var miners = system.addShips("miner", 2, locationAt(270E3), 5E3);
        miners = miners.concat(system.addShips("miner", 5, locationAt(360E3)), 5E3);
        miners = miners.concat(system.addShips("miner", 5, locationAt(450E3)), 5E3);

        for (var i=0; i<miners.length; i++)
            if (miners[i].isValid)
            {
                miners[i].switchAI("route1UpsMinerAI.plist");
                miners[i].primaryRole = "ups-miner";
            }
    }
}

this.getTimeToGo = function (deadline)
{
	var timer = deadline - clock.minutes/24/60;
	var days = Math.floor(timer)
	var hours = Math.round((timer - days)*24)
	var hoursText = ""
	if (hours>1) hoursText = " " + hours +" hrs"; else if (hours==1) hoursText = " " + hours +" hr";
	if (days > 1) return "within " + days + " days" + hoursText;
	if (days === 1) return "within 1 day" + hoursText;
	if (days === 0 && hours > 0) return "within" + hoursText;
	if (days === 0) return "now";
	return "by yesterday";
}

this.increaseReputation = function (reputation)
{
        if (!this.ups_reputation) this.ups_reputation = 0
        this.ups_reputation += reputation;
        while (this.ups_reputation >=3)
        {
            player.increaseParcelReputation();
            this.ups_reputation -= 3;
        };
        while (this.ups_reputation <= -3)
        {
            player.decreaseParcelReputation();
            this.ups_reputation += 3;
        }
}

this._reputation = function ()
{
    return player.parcelReputation;
}

this.launchEarthquakeRescue = function (count)
{
    for (var i=0; i<count; i++)
    {
        var shuttle = player.ship.dockedStation.launchShipWithRole("shuttle");
        shuttle.displayName = shuttle.displayName + " (earthquake resque)";
        shuttle.switchAI("fallingShuttleAI.plist");
    }
}

this.upsMarkSystem = function (ID, oxp_name)
{
    mission.markSystem({
               system: ID,
               name: oxp_name,
               markerColor: "magentaColor",
               markerShape: "MARKER_X"
    })
}

this.upsUnmarkSystem = function (ID, oxp_name)
{
    mission.unmarkSystem({
               system: ID,
               name: oxp_name
    })
}

this.updateUpsMarkers = function ()
{
    // one-time-only function to update pre-Oolite 1.77 markers to the new structure
    if (worldScripts["ups_docs"].ups_dplanet != null && worldScripts["ups_docs"].ups_dplanet >= 0)
    {
        if (["SPECIAL_DELIVERY", "CHRONICLE_DELIVERY", "EARTHQUAKE_DELIVERY"].indexOf(worldScripts["ups_docs"].ups_docs) > -1)
        {
            this.checkUpsMarker(worldScripts["ups_docs"].ups_dplanet, "ups_docs");
        }
    }
    if (worldScripts["ups_parcel"].ups_pplanet != null && worldScripts["ups_parcel"].ups_pplanet >= 0)
    {
        if (["SPECIAL_DELIVERY", "MEDICIN_DELIVERY", "MEDICIN_DELIVERY_2", "ASTRO_ACCEPTED"].indexOf(worldScripts["ups_parcel"].ups_parcel) > -1)
        {
            this.checkUpsMarker(worldScripts["ups_parcel"].ups_pplanet, "ups_parcel");
        }
    }
    if (worldScripts["ups_container"].ups_cplanet != null && worldScripts["ups_container"].ups_cplanet >= 0)
    {
        if (["SEARCHING", "SEARCHING2", "SEARCHING3"].indexOf(worldScripts["ups_container"].ups_container) > -1)
        {
            this.checkUpsMarker(worldScripts["ups_container"].ups_cplanet, "ups_container");
        }
    }
    if (worldScripts["ups_slaves"].ups_slplanet != null && worldScripts["ups_slaves"].ups_slplanet >= 0)
    {
        if (["HOMELAND", "FINAL"].indexOf(worldScripts["ups_slaves"].ups_slaves) > -1)
        {
            this.checkUpsMarker(worldScripts["ups_slaves"].ups_slplanet, "ups_slaves");
        }
    }
    if (worldScripts["ups_slaves"].ups_sl2planet != null && worldScripts["ups_slaves"].ups_sl2planet >= 0)
    {
        if (["RESCUE"].indexOf(worldScripts["ups_slaves"].ups_slaves) > -1)
        {
            this.checkUpsMarker(worldScripts["ups_slaves"].ups_sl2planet, "ups_slaves");
        }
    }
    if (worldScripts["ups_sun"].ups_splanet != null && worldScripts["ups_sun"].ups_splanet >= 0)
    {
        if (["SMALL_DELIVERY", "RESCUE"].indexOf(worldScripts["ups_sun"].ups_sun) > -1)
        {
            this.checkUpsMarker(worldScripts["ups_sun"].ups_splanet, "ups_sun");
        }
    }
}

this.checkUpsMarker = function (ID, oxp_name)
{
    var markedSystems = mission.markedSystems;
    for (var i=0; i<markedSystems.length; i++)
    {
        if (markedSystems[i].system == ID && markedSystems[i].name == "__oolite_legacy_destinations")
        {
            mission.unmarkSystem(ID); // unmark legacy marker
            this.upsMarkSystem(ID, oxp_name);// add the new marker
        }
    }
}

// Code below is used by ship scripts
this.messageTime = clock.absoluteSeconds // for use of spacing NPC messages by ship scripts.

this.messageAllowed = function ()
{
    if ((clock.absoluteSeconds - this.messageTime) > 5)
    {
        this.messageTime = clock.absoluteSeconds;
		return true;
    }
	else return false;
}

// Code below is linked to ship scripts
this.sendUpsDocMessage = function ()
{
	if (worldScripts["ups_docs"].messageAllowed() && this.ship.target && this.ship.target.isPlayer)
		this.ship.commsMessage(expandDescription("[ups_secret_talk]"))
    // no messages within 5 secs to avoid screen clutter.
}

this._overlay = function() {
    if (player.ship.hudHidden || player.ship.hudAllowsBigGui) {
        return {name:"upsnews-large.png", height:546};
    } else {
        return {name:"upsnews-small.png", height:546};
    }
}

this.cleanUpMarkers = function() {
    var ended = ["NO", "NOT_NOW"];
    var marked = mission.markedSystems;
    for (var i = marked.length - 1; i >= 0; i--) {
        if (marked[i].name === "ups_docs" && ended.indexOf(missionVariables.ups_docs) >= 0) mission.unmarkSystem({system:marked[i].system, name:"ups_docs"});
        if (marked[i].name === "ups_parcel" && ended.indexOf(missionVariables.ups_parcel) >= 0) mission.unmarkSystem({system:marked[i].system, name:"ups_parcel"});
        if (marked[i].name === "ups_container" && ended.indexOf(missionVariables.ups_container) >= 0) mission.unmarkSystem({system:marked[i].system, name:"ups_container"});
        if (marked[i].name === "ups_slaves" && ended.indexOf(missionVariables.ups_slaves) >= 0) mission.unmarkSystem({system:marked[i].system, name:"ups_slaves"});
    }
}